﻿using System;
using DL.Core.Data.EntityFramework.Auditing;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DL.Core.Data.EntityFramework.Tests.Auditing
{
    [TestClass]
    public class AuditorFactoryBaseTests
    {
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void Ctor_DerivedClassAddsInvalidAuditor_ArgumentException()
        {
            new InvalidAuditorFactory();
        }

        [TestMethod]
        public void Ctor_DerivedClassAddsValidAuditor_NoException()
        {
            var factory = MockAuditorFactory.Instance;
            Assert.IsNotNull(factory);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void MakeAuditor_TypeNotFound_ArgumentOutOfRangeException()
        {
            var factory = MockAuditorFactory.Instance;
            factory.MakeAuditor(typeof(InvalidObject));
        }

        [TestMethod]
        public void MakeAuditor_ValidType_ObjectOfGivenType()
        {
            var factory = MockAuditorFactory.Instance;
            var auditor = factory.MakeAuditor(typeof(MockObject));
            Assert.IsInstanceOfType(auditor, typeof(MockAuditor));
        }

        private class InvalidObject
        {
        }

        private class InvalidAuditorFactory : AuditorFactoryBase
        {
            public InvalidAuditorFactory()
            {
                this.AddAuditor(typeof(MockObject), typeof(MockObject));
            }
        }
    }
}
