﻿using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;

namespace DL.Core.Data.EntityFramework.Auditing
{
    public class ChangeProcessor
    {
        private readonly DbContext context;

        private readonly AuditorFactoryBase auditorFactory;

        public ChangeProcessor(DbContext databaseContext, AuditorFactoryBase auditorFactoryToUse)
        {
            this.context = databaseContext;
            this.auditorFactory = auditorFactoryToUse;
        }

        public IEnumerable<AuditLog> ProcessChanges(string userToken)
        {
            var result = new List<AuditLog>();
            IEnumerable<DbEntityEntry> changedEntities = this.GetAuditableChanges();

            foreach (var entity in changedEntities)
            {
                IAuditor auditor = this.auditorFactory.MakeAuditor(entity.GetType());
                IEnumerable<AuditLog> changes = auditor.AuditChange(entity, userToken);
                
                if (changes != null)
                {
                    result.AddRange(changes);    
                }
            }

            return result;
        }

        private IEnumerable<DbEntityEntry> GetAuditableChanges()
        {
            return this.context.ChangeTracker.Entries()
                .Where(p => p.State == EntityState.Added
                    || p.State == EntityState.Deleted
                    || p.State == EntityState.Modified);
        }
    }
}
