﻿using System;
using System.Collections.Generic;
using System.Linq;

using DL.Core.Enums;

using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DL.Core.Tests.Enums
{
    [TestClass]
    public class EnumUtilTests
    {
        #region Enums

        private enum ColorsWithDescriptions
        {
            [System.ComponentModel.Description("Maroon")]
            Red = 1,
            [System.ComponentModel.Description("Hunter Green")]
            Green = 2,
            [System.ComponentModel.Description("Navy Blue")]
            Blue = 3
        }

        private enum ColorsNoDescriptions
        {
            Red = 1,
            Green = 2,
            Blue = 3
        }

        #endregion

        #region ConvertValuesToListItemData() Tests

        //[TestMethod]
        //public void ConvertValuesToListItemDataEnumWithoutDescription()
        //{
        //    IEnumerable<ListItemData> data = EnumUtil.ConvertValuesToListItemData(typeof(ColorsNoDescriptions));
        //    Assert.AreEqual(3, data.Count());
        //    Assert.AreEqual("Red", data.First().Display);
        //    Assert.AreEqual("Blue", data.Last().Display);
        //}

        //[TestMethod]
        //public void ConvertValuesToListItemDataEnumWithDescription()
        //{
        //    IEnumerable<ListItemData> data = EnumUtil.ConvertValuesToListItemData(typeof(ColorsWithDescriptions));
        //    Assert.AreEqual(3, data.Count());
        //    Assert.AreEqual("Maroon", data.First().Display);
        //    Assert.AreEqual("Navy Blue", data.Last().Display);
        //}

        #endregion

        #region GetByByte() Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetByByteInvalidLowOrdinalThrowsException()
        {
            EnumUtil.GetByByte<ColorsNoDescriptions>(0);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetByByteInvalidHighOrdinalThrowsException()
        {
            EnumUtil.GetByByte<ColorsNoDescriptions>(5);
        }

        [TestMethod]
        public void GetByByteValidOrdinal()
        {
            var actual = EnumUtil.GetByByte<ColorsNoDescriptions>(2);
            Assert.AreEqual(ColorsNoDescriptions.Green, actual);
        }

        #endregion

        #region GetByte() Tests

        [TestMethod]
        public void GetByteValidEnumValue()
        {
            int actual = EnumUtil.GetByte(ColorsNoDescriptions.Green);
            Assert.AreEqual(2, actual);
        }

        #endregion

        #region GetByInteger() Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetByIntegerInvalidLowOrdinalThrowsException()
        {
            EnumUtil.GetByInteger<ColorsNoDescriptions>(0);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetByIntegerInvalidHighOrdinalThrowsException()
        {
            EnumUtil.GetByInteger<ColorsNoDescriptions>(5);
        }

        [TestMethod]
        public void GetByIntegerValidOrdinal()
        {
            var actual = EnumUtil.GetByInteger<ColorsNoDescriptions>(2);
            Assert.AreEqual(ColorsNoDescriptions.Green, actual);
        }

        #endregion

        #region GetInteger() Tests

        [TestMethod]
        public void GetIntegerValidEnumValue()
        {
            int actual = EnumUtil.GetInteger(ColorsNoDescriptions.Green);
            Assert.AreEqual(2, actual);
        }

        #endregion

        #region GetTitle() Tests

        [TestMethod]
        public void GetTitleEnumWitDescriptions()
        {
            string actual = EnumUtil.GetTitle(ColorsWithDescriptions.Blue);
            Assert.AreEqual("Navy Blue", actual);
        }

        [TestMethod]
        public void GetTitleEnumWithoutDescriptions()
        {
            string actual = EnumUtil.GetTitle(ColorsNoDescriptions.Blue);
            Assert.AreEqual("Blue", actual);
        }

        [TestMethod]
        public void GetTitleByIntegerEnumWitDescriptions()
        {
            string actual = EnumUtil.GetTitle<ColorsWithDescriptions>(3);
            Assert.AreEqual("Navy Blue", actual);
        }

        [TestMethod]
        public void GetTitleByIntegerEnumWithoutDescriptions()
        {
            string actual = EnumUtil.GetTitle<ColorsNoDescriptions>(3);
            Assert.AreEqual("Blue", actual);
        }

        #endregion

        #region IsValidEnumByte() Tests

        [TestMethod]
        public void IsValidEnumByteValidNumberTrue()
        {
            bool actual = EnumUtil.IsValidEnumByte<ColorsNoDescriptions>(2);
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void IsValidEnumByteInvalidNumberFalse()
        {
            bool actual = EnumUtil.IsValidEnumByte<ColorsNoDescriptions>(5);
            Assert.IsFalse(actual);
        }

        [TestMethod]
        public void IsValidEnumByteZeroFalse()
        {
            bool actual = EnumUtil.IsValidEnumByte<ColorsNoDescriptions>(0);
            Assert.IsFalse(actual);
        }

        #endregion

        #region IsValidEnumInteger() Tests

        [TestMethod]
        public void IsValidEnumIntegerValidNumberTrue()
        {
            bool actual = EnumUtil.IsValidEnumInteger<ColorsNoDescriptions>(2);
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void IsValidEnumIntegerInvalidNumberFalse()
        {
            bool actual = EnumUtil.IsValidEnumInteger<ColorsNoDescriptions>(5);
            Assert.IsFalse(actual);
        }

        [TestMethod]
        public void IsValidEnumIntegerNegativeNumberFalse()
        {
            bool actual = EnumUtil.IsValidEnumInteger<ColorsNoDescriptions>(-1);
            Assert.IsFalse(actual);
        }

        [TestMethod]
        public void IsValidEnumIntegerZeroFalse()
        {
            bool actual = EnumUtil.IsValidEnumInteger<ColorsNoDescriptions>(0);
            Assert.IsFalse(actual);
        }

        #endregion
    }
}
