﻿using System;
using System.Linq.Expressions;
using DL.Core.Expressions;
using DL.Core.Tests.Mocks;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DL.Core.Tests.Expressions
{
    [TestClass]
    public class BinaryExpressionBuilderTests
    {
        private BinaryExpressionBuilder builder;

        [TestInitialize]
        public void TestInitialize()
        {
            ParameterExpression parameterExpression = Expression.Parameter(typeof(SimpleObject), "x");
            this.builder = new BinaryExpressionBuilder(parameterExpression);
        }

        [TestCleanup]
        public void TestCleanup()
        {
            this.builder = null;
        }

        #region GetEqualsExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetEqualsExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            object filterValue = 2;
            this.builder.GetEqualsExpression(propertyName, filterValue);
        }

        [TestMethod]
        public void GetEqualsExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Id";
            object filterValue = 2;

            BinaryExpression expression = this.builder.GetEqualsExpression(propertyName, filterValue);
            Assert.AreEqual("(x.Id == 2)", expression.ToString());
        }

        #endregion

        #region GetGreaterThanExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetGreaterThanExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            object filterValue = 2;
            this.builder.GetGreaterThanExpression(propertyName, filterValue);
        }

        [TestMethod]
        public void GetGreaterThanExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Id";
            object filterValue = 2;

            BinaryExpression expression = this.builder.GetGreaterThanExpression(propertyName, filterValue);
            Assert.AreEqual("(x.Id > 2)", expression.ToString());
        }

        #endregion

        #region GetGreaterThanOrEqualExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetGreaterThanOrEqualExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            object filterValue = 3;
            this.builder.GetGreaterThanOrEqualExpression(propertyName, filterValue);
        }

        [TestMethod]
        public void GetGreaterThanOrEqualExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Id";
            object filterValue = 10;
            BinaryExpression expression = this.builder.GetGreaterThanOrEqualExpression(propertyName, filterValue);
            Assert.AreEqual("(x.Id >= 10)", expression.ToString());
        }

        #endregion

        #region GetLessThanExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetLessThanExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            object filterValue = 3;
            this.builder.GetLessThanExpression(propertyName, filterValue);
        }

        [TestMethod]
        public void GetLessThanExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Id";
            object filterValue = 0;

            BinaryExpression expression = this.builder.GetLessThanExpression(propertyName, filterValue);
            Assert.AreEqual("(x.Id < 0)", expression.ToString());
        }

        #endregion

        #region GetLessThanOrEqualExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetLessThanOrEqualExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            object filterValue = 2;
            this.builder.GetLessThanOrEqualExpression(propertyName, filterValue);
        }

        [TestMethod]
        public void GetLessThanOrEqualExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Id";
            object filterValue = 0;
            BinaryExpression expression = this.builder.GetLessThanOrEqualExpression(propertyName, filterValue);
            Assert.AreEqual("(x.Id <= 0)", expression.ToString());
        }

        #endregion

        #region GetBetweenExclusiveExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetBetweenExclusiveExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            object filterValue = 2;
            object filterValueTwo = 4;
            this.builder.GetBetweenExclusiveExpression(propertyName, filterValue, filterValueTwo);
        }

        [TestMethod]
        public void GetBetweenExclusiveExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Id";
            object filterValue = 10;
            object filterValueTwo = 20;
            BinaryExpression expression = this.builder.GetBetweenExclusiveExpression(propertyName, filterValue, filterValueTwo);
            Assert.AreEqual("((x.Id > 10) And (x.Id < 20))", expression.ToString());
        }

        #endregion

        #region GetBetweenInclusiveExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetBetweenInclusiveExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            object filterValue = 2;
            object filterValueTwo = 4;
            this.builder.GetBetweenInclusiveExpression(propertyName, filterValue, filterValueTwo);
        }

        [TestMethod]
        public void GetBetweenInclusiveExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Id";
            object filterValue = 10;
            object filterValueTwo = 20;
            BinaryExpression expression = this.builder.GetBetweenInclusiveExpression(propertyName, filterValue, filterValueTwo);
            Assert.AreEqual("((x.Id >= 10) And (x.Id <= 20))", expression.ToString());
        }

        #endregion
    }
}
