﻿using System;
using System.Linq.Expressions;
using DL.Core.Expressions;
using DL.Core.Tests.Mocks;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DL.Core.Tests.Expressions
{
    [TestClass]
    public class MethodCallExpressionBuilderTests
    {
        private MethodCallExpressionBuilder builder;

        [TestInitialize]
        public void TestInitialize()
        {
            ParameterExpression parameterExpression = Expression.Parameter(typeof(SimpleObject), "x");
            this.builder = new MethodCallExpressionBuilder(parameterExpression);
        }

        [TestCleanup]
        public void TestCleanup()
        {
            this.builder = null;
        }

        #region GetContainsExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetContainsExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            const string filterValue = "NA";
            this.builder.GetContainsExpression(propertyName, filterValue);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetContainsExpression_InvalidPropertyType_ArgumentException()
        {
            const string propertyName = "Id";
            const string filterValue = "NA";
            this.builder.GetContainsExpression(propertyName, filterValue);
        }

        [TestMethod]
        public void GetContainsExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Title";
            const string filterValue = "2";

            MethodCallExpression expression = this.builder.GetContainsExpression(propertyName, filterValue);
            Assert.AreEqual("x.Title.Contains(\"2\")", expression.ToString());
        }

        #endregion

        #region GetEndsWithExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetEndsWithExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            const string filterValue = "NA";
            this.builder.GetEndsWithExpression(propertyName, filterValue);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetEndsWithExpression_InvalidPropertyType_ArgumentException()
        {
            const string propertyName = "Id";
            const string filterValue = "NA";
            this.builder.GetEndsWithExpression(propertyName, filterValue);
        }

        [TestMethod]
        public void GetEndsWithExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Title";
            const string filterValue = "2";

            MethodCallExpression expression = this.builder.GetEndsWithExpression(propertyName, filterValue);
            Assert.AreEqual("x.Title.EndsWith(\"2\")", expression.ToString());
        }

        #endregion

        #region GetStartsWithExpression Tests

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetStartsWithExpression_InvalidProperty_ArgumentException()
        {
            const string propertyName = "InvalidProperty";
            const string filterValue = "NA";
            this.builder.GetStartsWithExpression(propertyName, filterValue);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetStartsWithExpression_InvalidPropertyType_ArgumentException()
        {
            const string propertyName = "Id";
            const string filterValue = "NA";
            this.builder.GetStartsWithExpression(propertyName, filterValue);
        }

        [TestMethod]
        public void GetStartsWithExpression_ValidPropertyAndFilterData_ValidExpresssion()
        {
            const string propertyName = "Title";
            const string filterValue = "Title 2";

            MethodCallExpression expression = this.builder.GetStartsWithExpression(propertyName, filterValue);
            Assert.AreEqual("x.Title.StartsWith(\"Title 2\")", expression.ToString());
        }

        #endregion
    }
}
