﻿using System.Collections.Generic;
using FluentValidation;
using FluentValidation.Results;

namespace DL.Core.Validation.Fluent
{
    public class FluentValidationService<T> : IValidationService<T> where T : class
    {
        private readonly IValidator validator;

        /// <summary>
        /// Initializes a new instance of the FluentValidationService class.
        /// </summary>
        /// <param name="validator">Validator to use in validation. Must implement IValidator.</param>
        public FluentValidationService(IValidator validator)
        {
            this.validator = validator;
        }

        public void Validate(T entity)
        {
            ValidationResult results = this.validator.Validate(entity);

            if (!results.IsValid)
            {
                ValidationException exception = this.BuildException(results.Errors);
                throw exception;
            }
        }

        private ValidationException BuildException(IEnumerable<ValidationFailure> failures)
        {
            var exception = new ValidationException("Validation failed.");

            foreach (var failure in failures)
            {
                var error = new ValidationError
                {
                    ErrorMessage = failure.ErrorMessage,
                    PropertyName = failure.PropertyName
                };
                exception.ValidationErrors.Add(error);
            }

            return exception;
        }
    }
}
