﻿using System.Data.Entity;
using System.Data.Entity.ModelConfiguration;

namespace DL.Util.Web.Security.EntityFramework
{
    public class ProviderContext : DbContext
    {
        public ProviderContext(string connectionStringName)
            : base(connectionStringName)
        {
        }

        public DbSet<Role> Roles { get; set; }

        public DbSet<User> Users { get; set; }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            EntityTypeConfiguration<User> userEntity = modelBuilder.Entity<User>();
            userEntity.ToTable("User");
            userEntity.HasKey(x => x.Id);
            userEntity.Property(x => x.UserName).HasMaxLength(100);
            userEntity.Property(x => x.Email).HasMaxLength(255);

            EntityTypeConfiguration<Role> roleEntity = modelBuilder.Entity<Role>();
            roleEntity.ToTable("Role");
            roleEntity.HasKey(x => x.Id);
            roleEntity.Property(x => x.Name).HasMaxLength(100);

            userEntity
                .HasMany(u => u.Roles)
                .WithMany(r => r.Users)
                .Map(m =>
                    {
                        m.MapLeftKey("UserId");
                        m.MapRightKey("RoleId");
                        m.ToTable("UserRole");
                    });
        }
    }
}
