﻿using System.Net;
using System.Security.Principal;
using Microsoft.Reporting.WebForms;

namespace DL.Core.Web.Reporting
{
    /// <summary>
    /// Class which allows the passing of credentials to ASP.NET report viewer controls.
    /// </summary>
    public class ReportViewerCredentials : IReportServerCredentials
    {
        private readonly string userName;

        private readonly string password;

        private readonly string domain;

        /// <summary>
        /// Initializes a new instance of the <see cref="ReportViewerCredentials"/> class.
        /// </summary>
        /// <param name="userNameToUse">User name.</param>
        /// <param name="passwordToUse">Password for the user.</param>
        /// <param name="domainToUse">Domain in which the user belongs.</param>
        public ReportViewerCredentials(string userNameToUse, string passwordToUse, string domainToUse)
        {
            this.userName = userNameToUse;
            this.password = passwordToUse;
            this.domain = domainToUse;
        }

        public WindowsIdentity ImpersonationUser
        {
            get { return null; }
        }

        public ICredentials NetworkCredentials
        {
            get { return new NetworkCredential(this.userName, this.password, this.domain); }
        }

        public bool GetFormsCredentials(
            out Cookie authCookie, out string userNameToUse, out string passwordToUse, out string authority)
        {
            authCookie = null;
            userNameToUse = passwordToUse = authority = null;
            return false;
        }
    }
}
