﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.Reporting.WebForms;

namespace DL.Core.Web.Reporting
{
    public class ReportViewerHelper
    {
        private readonly Dictionary<string, string> parameters;

        /// <summary>
        /// Initializes a new instance of the <see cref="ReportViewerHelper"/> class.
        /// </summary>
        public ReportViewerHelper()
        {
            this.parameters = new Dictionary<string, string>();
        }

        /// <summary>
        /// Gets or sets the path of the reporting server report.
        /// </summary>
        public string ReportPath { get; set; }

        /// <summary>
        /// Gets or sets the title of the reporting server report.
        /// </summary>
        public string ReportTitle { get; set; }

        /// <summary>
        /// Adds a parameter to be used for the report.
        /// </summary>
        /// <param name="name">Name of the report parameter.</param>
        /// <param name="value">Value of the report parameter.</param>
        public void AddParameter(string name, string value)
        {
            this.parameters.Add(name, value);
        }

        /// <summary>
        /// Gets a list of parameters for the reporting server report.
        /// </summary>
        /// <returns>List of ReportParameter object for the reporting server report.</returns>
        public IEnumerable<ReportParameter> GetReportParameters()
        {
            return this.parameters.Select(x => new ReportParameter(x.Key, x.Value)).ToList();
        }
    }
}
