﻿using System;
using System.Collections.Generic;

namespace DL.Core.Web.Rss
{
    /// <summary>
    /// Class which represents an RSS feed.
    /// </summary>
    public class Feed
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Feed"/> class.
        /// </summary>
        public Feed()
        {
            this.Image = new Image();
            this.Items = new List<Item>();
        }

        #region <channel> Properties

        /// <summary>
        /// Gets or sets the title property from the channel tag.
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        /// Gets or sets the link property from the channel tag.
        /// </summary>
        public string Link { get; set; }

        /// <summary>
        /// Gets or sets the description property from the channel tag.
        /// </summary>
        public string Description { get; set; }

        /// <summary>
        /// Gets or sets the last build date property from the channel tag.
        /// </summary>
        public DateTime LastBuildDate { get; set; }

        /// <summary>
        /// Gets or sets the language property from the channel tag.
        /// </summary>
        public string Language { get; set; }

        /// <summary>
        /// Gets or sets the generator property from the channel tag.
        /// </summary>
        public string Generator { get; set; }

        /// <summary>
        /// Gets or sets the image tag contained withing the channel tag.
        /// </summary>
        public Image Image { get; set; }

        /// <summary>
        /// Gets or sets the update period property from the channel tag.
        /// </summary>
        public string UpdatePeriod { get; set; }

        /// <summary>
        /// Gets or sets the update frequency property from the channel tag.
        /// </summary>
        public string UpdateFrequency { get; set; }

        #endregion 

        /// <summary>
        /// Gets or sets the list of item tags contained in the channel tag.
        /// </summary>
        public List<Item> Items { get; set; }
    }
}
