﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;

using DL.Core.Contracts;
using DL.Core.TypeUtilities;

namespace DL.Core.Web
{
    /// <summary>
    /// Generic boilerplate web control utility methods.
    /// </summary>
    public static class WebControlUtil
    {
        public static decimal? GetNullableDecimalFromTextControl(ITextControl control)
        {
            return GetNullableDecimalFromString(control.Text);
        }

        public static int? GetNullableIntegerFromDropDownList(DropDownList dropDownControl)
        {
            return GetNullableIntegerFromString(dropDownControl.SelectedValue);
        }

        public static int? GetNullableIntegerFromHiddenField(HiddenField control)
        {
            return GetNullableIntegerFromString(control.Value);
        }

        public static int? GetNullableIntegerFromTextControl(ITextControl control)
        {
            return GetNullableIntegerFromString(control.Text);
        }

        public static string GetStringFromTextControl(ITextControl textControl)
        {
            return textControl.Text.Trim();
        }

        public static void LoadList<T>(ListControl control, IEnumerable<ILabeledIdentifiable<T>> list, string firstItemText = "")
        {
            control.Items.Clear();

            if (!string.IsNullOrEmpty(firstItemText))
            {
                control.Items.Add(new ListItem { Text = firstItemText, Value = string.Empty });    
            }

            foreach (ILabeledIdentifiable<T> item in list.OrderBy(x => x.Title))
            {
                control.Items.Add(new ListItem { Text = item.Title, Value = item.Id.ToString() });
            }
        }

        private static decimal? GetNullableDecimalFromString(string value)
        {
            //var validator = new StringValidator(value.Trim());

            //if (string.IsNullOrEmpty(value)
            //    || !validator.IsNumeric)
            //{
                return null;
            //}

            //return Convert.ToDecimal(value);
        }

        private static int? GetNullableIntegerFromString(string value)
        {
            return !string.IsNullOrEmpty(value)
                ? Convert.ToInt32(value)
                : (int?)null;
        }
    }
}
