﻿using System;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

namespace DL.Core.Web.WebParts
{
    /// <summary>
    /// EditorPart for the HelloWorldWebPart.
    /// </summary>
    internal sealed class HelloWorldEditorPart : EditorPart
    {
        /// <summary>
        /// Initializes a new instance of the HelloWorldEditorPart class.
        /// </summary>
        /// <param name="id">ID of associated HelloWorldWebPart object.</param>
        public HelloWorldEditorPart(string id)
        {
            // editor parts must have unique id per web part to ensure that multiple
            // web parts of the same type can be used on a page
            this.ID = string.Format("HelloWorldEditorPart_{0}", id);
            this.Title = "Hello World Options";
        }

        /// <summary>
        /// Gets or sets the TextBox for the message.
        /// </summary>
        private TextBox MessageTextBox { get; set; }

        /// <summary>
        /// Gets or sets the ValidationSummary for the EditorPart.
        /// </summary>
        private ValidationSummary ValidationSummary { get; set; }

        /// <summary>
        /// Places data in editor part properties/controls into web part properties.
        /// </summary>
        /// <returns>True if successful.</returns>
        public override bool ApplyChanges()
        {
            this.EnsureChildControls();
            var webPart = WebPartToEdit as HelloWorldWebPart;
            if (webPart != null)
            {
                webPart.Message = this.MessageTextBox.Text;
            }

            return true;
        }

        /// <summary>
        /// Places data in web part properties into editor part properties/controls
        /// </summary>
        public override void SyncChanges()
        {
            this.EnsureChildControls();
            var webPart = WebPartToEdit as HelloWorldWebPart;
            if (webPart != null)
            {
                this.MessageTextBox.Text = webPart.Message;
            }
        }

        /// <summary>
        /// CreateChildControls override.
        /// </summary>
        protected override void CreateChildControls()
        {
            try
            {
                base.CreateChildControls();

                var pnlGroup = new Panel(); // configuration section panel

                this.ValidationSummary = new ValidationSummary
                    {
                        ID = "ValidationSummary",
                        DisplayMode = ValidationSummaryDisplayMode.SingleParagraph,
                        ShowSummary = true
                    };
                pnlGroup.Controls.Add(this.ValidationSummary);

                this.MessageTextBox = new TextBox { ID = "MessageTextBox" };

                var webPart = WebPartToEdit as HelloWorldWebPart;

                if (webPart != null)
                {
                    var lblMessage = new Label { Text = "Message" };

                    var pnlMessageLabel = new Panel(); // head for configuration section
                    pnlMessageLabel.Controls.Add(lblMessage);

                    // Required field validator
                    var messageRequiredValidator = new RequiredFieldValidator
                        {
                            ControlToValidate = this.MessageTextBox.ClientID,
                            ErrorMessage = "Message cannot be blank",
                            Display = ValidatorDisplay.Dynamic,
                            Text = "*"
                        };

                    var pnlMessageControls = new Panel(); // body of configuration section
                    pnlMessageControls.Controls.Add(this.MessageTextBox);
                    pnlMessageControls.Controls.Add(messageRequiredValidator);

                    pnlGroup.Controls.Add(pnlMessageLabel);
                    pnlGroup.Controls.Add(pnlMessageControls);
                }

                Controls.Add(pnlGroup);
            }
            catch (Exception ex)
            {
                WebPartUtil.HandleException(this, ex);
            }
        }
    }
}
