﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;

namespace DL.Core.Web.WebParts
{
    /// <summary>
    /// WebPart sample class.
    /// </summary>
    [DefaultProperty("Message")]
    [ToolboxData("<{0}:HelloWorldWebPart runat=server></{0}:HelloWorldWebPart>")]
    internal sealed class HelloWorldWebPart : WebPart
    {
        /// <summary>
        /// Message field.
        /// </summary>
        private string message;

        /// <summary>
        /// Initializes a new instance of the HelloWorldWebPart class.
        /// </summary>
        public HelloWorldWebPart()
        {
            this.ExportMode = WebPartExportMode.All;
        }

        ////[System.ComponentModel.Bindable(true)]
        ////[System.ComponentModel.Category("My Property Group")]
        ////[WebDisplayName("MyProperty")]
        ////[WebDescription("Meaningless Property")]
        ////[PersistenceMode(PersistenceMode.InnerProperty)]

        /// <summary>
        /// Gets or sets the message property.
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        public string Message
        {
            get { return this.message ?? (this.message = "Hello World!"); }
            set { this.message = value; }
        }

        /// <summary>
        /// Gets or sets the message LiteralControl.
        /// </summary>
        private LiteralControl MessageLiteral { get; set; }

        /// <summary>
        /// CreateEditorParts override.
        /// </summary>
        /// <returns>Customized EditorPartCollection</returns>
        public override EditorPartCollection CreateEditorParts()
        {
            try
            {
                List<EditorPart> editorParts 
                    = new List<EditorPart> { new HelloWorldEditorPart(this.ID) };
                return new EditorPartCollection(editorParts);
            }
            catch (Exception ex)
            {
                WebPartUtil.HandleException(this, ex);
                return base.CreateEditorParts();
            }
        }

        /// <summary>
        /// CreateChildControls override.
        /// </summary>
        protected override void CreateChildControls()
        {
            try
            {
                base.CreateChildControls();
                this.MessageLiteral = new LiteralControl { Text = this.Message };
                Controls.Add(this.MessageLiteral);
            }
            catch (Exception ex)
            {
                WebPartUtil.HandleException(this, ex);
            }
        }

        /// <summary>
        /// OnPreRender override.
        /// </summary>
        /// <param name="e">EventArgs for the override.</param>
        protected override void OnPreRender(EventArgs e)
        {
            try
            {
                base.OnPreRender(e);
                this.EnsureChildControls();
                this.MessageLiteral.Text = this.Message;
            }
            catch (Exception ex)
            {
                WebPartUtil.HandleException(this, ex);
            }
        }
    }
}
