﻿using System.DirectoryServices.AccountManagement;

namespace DL.Core.DirectoryServices
{
    /// <summary>
    /// Class which performs management operations against Active Directory accounts.
    /// </summary>
    public class AccountManager
    {
        private readonly string path;
        private readonly string organizationalUnit;

        public AccountManager(string pathToUse, string organizationalUnitToUse)
        {
            this.path = pathToUse;
            this.organizationalUnit = organizationalUnitToUse;
        }

        public bool IsAuthenticated(string username, string password)
        {
            using (var context = new PrincipalContext(ContextType.Domain, this.path, this.organizationalUnit))
            {
                return context.ValidateCredentials(username, password);
            }
        }

        public void ChangePassword(string username, string oldPassword, string newPassword)
        {
            try
            {
                using (var context = new PrincipalContext(ContextType.Domain, this.path, username, oldPassword))
                {
                    using (var user = UserPrincipal.FindByIdentity(context, username))
                    {
                        user.ChangePassword(oldPassword, newPassword);
                    }
                }
            }
            catch (PasswordException)
            {
                throw new InvalidPasswordException();
            }
        }
    }
}
