﻿using System;

namespace DL.Core.DirectoryServices
{
    /// <summary>
    /// Class which defines an Active Directory exception.
    /// </summary>
    [Serializable]
    public class InvalidPasswordException : Exception
    {
        private const string defaultMessage =
            "Password does not meet minimum security requirements. Password must be at least 8 characters long, contain capital letters, lower case letters, and numbers. Also, you cannot change the password back to something it was recently.";

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidPasswordException"/> class.
        /// </summary>
        public InvalidPasswordException()
            : base(defaultMessage)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidPasswordException"/> class.
        /// </summary>
        /// <param name="errorMessage">Message to assign to the exception.</param>
        public InvalidPasswordException(string errorMessage)
            : base(errorMessage)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidPasswordException"/> class.
        /// </summary>
        /// <param name="errorMessage">Message to assign to the exception.</param>
        /// <param name="innerException">The original exception.</param>
        public InvalidPasswordException(string errorMessage, Exception innerException)
            : base(errorMessage, innerException)
        {
        }
    }
}
