﻿using System.Linq.Expressions;

namespace DL.Core.Expressions
{
    internal class BinaryExpressionBuilder
    {
        private readonly ParameterExpression parameterExpression;

        public BinaryExpressionBuilder(ParameterExpression parameterExpressionToUse)
        {
            this.parameterExpression = parameterExpressionToUse;
        }

        #region Public Methods - Filter Type Binary Expression Builders

        public BinaryExpression GetBetweenExclusiveExpression(
            string propertyName, object leftCompareValue, object rightCompareValue)
        {
            var leftExpression = this.GetGreaterThanExpression(propertyName, leftCompareValue);
            var rightExpression = this.GetLessThanExpression(propertyName, rightCompareValue);
            return Expression.And(leftExpression, rightExpression);
        }

        public BinaryExpression GetBetweenInclusiveExpression(
            string propertyName, object leftCompareValue, object rightCompareValue)
        {
            var leftExpression = this.GetGreaterThanOrEqualExpression(propertyName, leftCompareValue);
            var rightExpression = this.GetLessThanOrEqualExpression(propertyName, rightCompareValue);
            return Expression.And(leftExpression, rightExpression);
        }

        public BinaryExpression GetEqualsExpression(string propertyName, object compareValue)
        {
            var memberExpression = this.BuildPropertyExpression(propertyName);
            var constantExpression = this.BuildConstantExpression(compareValue);
            return Expression.Equal(memberExpression, constantExpression);
        }

        public BinaryExpression GetGreaterThanExpression(string propertyName, object compareValue)
        {
            var memberExpression = this.BuildPropertyExpression(propertyName);
            var constantExpression = this.BuildConstantExpression(compareValue);
            return Expression.GreaterThan(memberExpression, constantExpression);
        }

        public BinaryExpression GetGreaterThanOrEqualExpression(string propertyName, object compareValue)
        {
            var memberExpression = this.BuildPropertyExpression(propertyName);
            var constantExpression = this.BuildConstantExpression(compareValue);
            return Expression.GreaterThanOrEqual(memberExpression, constantExpression);
        }

        public BinaryExpression GetLessThanExpression(string propertyName, object compareValue)
        {
            var memberExpression = this.BuildPropertyExpression(propertyName);
            var constantExpression = this.BuildConstantExpression(compareValue);
            return Expression.LessThan(memberExpression, constantExpression);
        }

        public BinaryExpression GetLessThanOrEqualExpression(string propertyName, object compareValue)
        {
            var memberExpression = this.BuildPropertyExpression(propertyName);
            var constantExpression = this.BuildConstantExpression(compareValue);
            return Expression.LessThanOrEqual(memberExpression, constantExpression);
        }

        #endregion

        #region Private Methods

        private ConstantExpression BuildConstantExpression(object value)
        {
            return Expression.Constant(value);
        }

        private MemberExpression BuildPropertyExpression(string propertyName)
        {
            return Expression.Property(this.parameterExpression, propertyName);
        }

        #endregion
    }
}
