﻿namespace DL.Core.Expressions
{
    /// <summary>
    /// Class which uses the System.Linq.Expressions namespace to dynamically build
    /// expressions for advanced querying interfaces.
    /// </summary>
    public class ExpressionFilter
    {
        /// <summary>
        /// Initializes a new instance of the ExpressionFilter class.
        /// </summary>
        public ExpressionFilter()
        {
            this.FilterType = ExpressionFilterType.Equals;
            this.CompositeType = ExpressionCompositeType.And;
        }

        /// <summary>
        /// Gets or sets the filter type.
        /// </summary>
        public ExpressionFilterType FilterType { get; set; }

        /// <summary>
        /// Gets or sets the composite type of the filter such as And or Or.
        /// </summary>
        public ExpressionCompositeType CompositeType { get; set; }

        /// <summary>
        /// Gets or sets the value to filter by.
        /// </summary>
        public object FilterValue { get; set; }

        /// <summary>
        /// Gets or sets the second value to filter by.
        /// Should only be used with one of the Between FilterTypes.
        /// </summary>
        public object FilterValueTwo { get; set; }

        /// <summary>
        /// Gets or sets the name of the property to filter by.
        /// </summary>
        public string PropertyName { get; set; }
    }
}
