﻿using System;
using System.Linq.Expressions;
using System.Reflection;

namespace DL.Core.Expressions
{
    internal class MethodCallExpressionBuilder
    {
        private readonly ParameterExpression parameterExpression;

        public MethodCallExpressionBuilder(ParameterExpression parameterExpressionToUse)
        {
            this.parameterExpression = parameterExpressionToUse;
        }

        public MethodCallExpression GetContainsExpression(string propertyName, string compareValue)
        {
            return this.BuildStringMethodCallExpression("Contains", propertyName, compareValue);
        }

        public MethodCallExpression GetEndsWithExpression(string propertyName, string compareValue)
        {
            return this.BuildStringMethodCallExpression("EndsWith", propertyName, compareValue);
        }

        public MethodCallExpression GetStartsWithExpression(string propertyName, string compareValue)
        {
            return this.BuildStringMethodCallExpression("StartsWith", propertyName, compareValue);
        }

        private MethodCallExpression BuildMethodCallExpression(
            Type methodOwningType, string methodName, Type[] parameterTypes, string propertyName, string compareValue)
        {
            MemberExpression memberExpression = Expression.Property(this.parameterExpression, propertyName);
            ConstantExpression constantExpression = Expression.Constant(compareValue);

            MethodInfo method = methodOwningType.GetMethod(methodName, parameterTypes);
            return Expression.Call(memberExpression, method, constantExpression);
        }

        private MethodCallExpression BuildStringMethodCallExpression(string methodName, string propertyName, string compareValue)
        {
            Type stringType = typeof(string);
            return this.BuildMethodCallExpression(stringType, methodName, new[] { stringType }, propertyName, compareValue);
        }
    }
}
