﻿using System.IO;
using System.IO.Compression;

namespace DL.Core.IO.Compression
{
    /// <summary>
    /// Class which contains utility methods for GZip compress.
    /// </summary>
    public static class GZipUtil
    {
        /// <summary>Compresses a file using GZip compression.</summary>
        /// <param name="inFileName">File name of original file.</param>
        /// <param name="outFileName">File name to give to compressed file.</param>
        public static void GZipCompressFile(string inFileName, string outFileName)
        {
            using (FileStream sourceFile = File.OpenRead(inFileName))
            using (FileStream destFile = File.Create(outFileName))
            using (var compStream = new GZipStream(destFile, CompressionMode.Compress))
            {
                int theByte = sourceFile.ReadByte();

                while (theByte != -1)
                {
                    compStream.WriteByte((byte)theByte);
                    theByte = sourceFile.ReadByte();
                }    
            }
        }

        /// <summary>Decompresses a file which was compressed using GZip compression.</summary>
        /// <param name="inFileName">File name of compressed file.</param>
        /// <param name="outFileName">File name to give to decompressed file.</param>
        public static void GZipDecompressFile(string inFileName, string outFileName)
        {
            using (FileStream sourceFile = File.OpenRead(inFileName))
            using (FileStream destFile = File.Create(outFileName))
            using (var compStream = new GZipStream(sourceFile, CompressionMode.Decompress))
            {
                int theByte = compStream.ReadByte();

                while (theByte != -1)
                {
                    destFile.WriteByte((byte)theByte);
                    theByte = compStream.ReadByte();
                }    
            }
        }
    }
}
