using System;
using System.Collections.Generic;

namespace DL.Core
{
    /// <summary>
    /// Implementation of Service Locator design pattern.
    /// </summary>
    public abstract class ServiceLocatorBase : IServiceLocator
    {
        private readonly Dictionary<Type, object> services;

        protected ServiceLocatorBase()
        {
            this.services = new Dictionary<Type, object>();
        }

        /// <summary>
        /// Locates and returns a service if registered.
        /// </summary>
        /// <typeparam name="T">Interface type to find.</typeparam>
        /// <returns>Service of type T if found. Otherwise, KeyNotFoundException.</returns>
        public T Locate<T>()
        {
            return (T)this.services[typeof(T)];
        }

        /// <summary>
        /// Registers a service of the given type.
        /// </summary>
        /// <typeparam name="T">Interface type of the service.</typeparam>
        /// <param name="service">Service to register.</param>
        public void Register<T>(T service)
        {
            this.services.Add(typeof(T), service);
        }

        /// <summary>
        /// Unregisters, or removes, a service from the Service Locator.
        /// </summary>
        /// <typeparam name="T">Interface type to remove.</typeparam>
        public void Unregister<T>()
        {
            Type type = typeof(T);
            if (this.services.ContainsKey(type))
            {
                this.services.Remove(type);
            }
        }
    }
}