﻿using System;

namespace DL.Core.TypeUtilities
{
    /// <summary>
    /// Class which contains DateTime utility methods.
    /// </summary>
    public static class DateTimeConverter
    {
        #region DateTime to String Conversion Methods

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// full date and long time string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: Sunday, January 31, 2010 12:45:30 PM</returns>
        public static string ConvertToFullDateLongTimeString(DateTime dateTime)
        {
            return string.Format("{0:f}", dateTime);
        }

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// full date and short time string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: Sunday, January 31, 2010 12:45 PM</returns>
        public static string ConvertToFullDateShortTimeString(DateTime dateTime)
        {
            return string.Format("{0:f}", dateTime);
        }

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// general date and short time string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: 1/31/2010 12:45 PM</returns>
        public static string ConvertToGeneralDateShortTimeString(DateTime dateTime)
        {
            return string.Format("{0:g}", dateTime);
        }

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// month and day string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: January 31</returns>
        public static string ConvertToMonthDayString(DateTime dateTime)
        {
            return string.Format("{0:M}", dateTime);
        }

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// RFC1123 string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: Sun, 31 Jan 2010 12:45:30 GMT</returns>
        public static string ConvertToRfc1123String(DateTime dateTime)
        {
            return string.Format("{0:R}", dateTime);
        }

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// sortable date and time string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: 2010-01-31T12:45:30</returns>
        public static string ConvertToSortableDateTimeString(DateTime dateTime)
        {
            return string.Format("{0:s}", dateTime);
        }

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// universal sortable date and time string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: 2010-01-31 12:45:30Z</returns>
        public static string ConvertToUniversalSortableString(DateTime dateTime)
        {
            return string.Format("{0:u}", dateTime);
        }

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// Coordinated Universal Time (UTC) string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: Sunday, January 31, 2010 6:45:30 PM
        /// (This takes into account time zone and returns UTC time)</returns>
        public static string ConvertToUniversalString(DateTime dateTime)
        {
            return string.Format("{0:U}", dateTime);
        }

        /// <summary>
        /// Converts the value of the current DateTime object to its equivalent 
        /// year and month string representation.
        /// </summary>
        /// <param name="dateTime">DateTime to convert to a string.</param>
        /// <returns>Date and time string in the format: January, 2010</returns>
        public static string ConvertToYearMonthString(DateTime dateTime)
        {
            return string.Format("{0:Y}", dateTime);
        }

        #endregion
    }
}
