﻿using System;
using System.Globalization;

namespace DL.Core.TypeUtilities
{
    /// <summary>DateTime helper and utility methods.</summary>
    public static class DateTimeUtil
    {
        /// <summary>
        /// Converts the Time Zone of a DateTime.
        /// </summary>
        /// <param name="dateToConvert">Date to convert.</param>
        /// <returns>The given date in Central Standard Time.</returns>
        public static DateTime ChangeTimeZone(DateTime dateToConvert)
        {
            TimeZoneInfo cstTimeZoneInfo = TimeZoneInfo.FindSystemTimeZoneById("Central Standard Time");
            return TimeZoneInfo.ConvertTimeFromUtc(dateToConvert, cstTimeZoneInfo);
        }

        /// <summary>
        /// Converts the Time Zone of a DateTime.
        /// </summary>
        /// <param name="dateToConvert">Date to convert.</param>
        /// <param name="timeZoneInfo">Time zone to convert the date to.</param>
        /// <returns>The given date in the given time zone.</returns>
        public static DateTime ChangeTimeZone(DateTime dateToConvert, TimeZoneInfo timeZoneInfo)
        {
            return TimeZoneInfo.ConvertTimeFromUtc(dateToConvert, timeZoneInfo);
        }
    }
}
