﻿using System;
using System.Linq;

namespace DL.Core
{
    /// <summary>
    /// Class which contains utility methods.
    /// </summary>
    public static class Util
    {
        /// <summary>
        /// Gets the signature of the calling function.
        /// </summary>
        /// <returns>Function signature.</returns>
        public static string GetMethodSignature()
        {
            string stackTrace = Environment.StackTrace.Split("\n".ToCharArray())[4];
            stackTrace = stackTrace.Replace("at ", string.Empty);
            stackTrace = stackTrace.Substring(0, stackTrace.IndexOf(" in "));
            return stackTrace.Trim();
        }

        /// <summary>
        /// Determines whether or not a type implements the given interface.
        /// </summary>
        /// <param name="type">Type object to check for the interface.</param>
        /// <param name="interfaceType">Interface Type object to find.</param>
        /// <returns>True if implemented, otherwise false.</returns>
        public static bool DoesTypeImplementInterface(Type type, Type interfaceType)
        {
            if (!interfaceType.IsInterface)
            {
                throw new ArgumentException("'interfaceType' must be an interface.", "interfaceType");
            }

            return type.GetInterfaces().Any(x => x.FullName == interfaceType.FullName);
        }
    }
}
